"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
from isolate.connections.grpc.definitions import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
from isolate.server.definitions import server_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class HostedRun(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENVIRONMENTS_FIELD_NUMBER: builtins.int
    MACHINE_REQUIREMENTS_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    @property
    def environments(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[server_pb2.EnvironmentDefinition]:
        """Environment definitions."""
    @property
    def machine_requirements(self) -> global___MachineRequirements:
        """Machine requirements"""
    @property
    def function(self) -> common_pb2.SerializedObject:
        """Function to run."""
    def __init__(
        self,
        *,
        environments: collections.abc.Iterable[server_pb2.EnvironmentDefinition] | None = ...,
        machine_requirements: global___MachineRequirements | None = ...,
        function: common_pb2.SerializedObject | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_machine_requirements", b"_machine_requirements", "function", b"function", "machine_requirements", b"machine_requirements"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_machine_requirements", b"_machine_requirements", "environments", b"environments", "function", b"function", "machine_requirements", b"machine_requirements"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_machine_requirements", b"_machine_requirements"]) -> typing_extensions.Literal["machine_requirements"] | None: ...

global___HostedRun = HostedRun

@typing_extensions.final
class HostedRunCron(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENVIRONMENTS_FIELD_NUMBER: builtins.int
    MACHINE_REQUIREMENTS_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    CRON_FIELD_NUMBER: builtins.int
    @property
    def environments(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[server_pb2.EnvironmentDefinition]:
        """Environment definitions."""
    @property
    def machine_requirements(self) -> global___MachineRequirements:
        """Machine requirements"""
    @property
    def function(self) -> common_pb2.SerializedObject:
        """Function to run."""
    cron: builtins.str
    """cron string to represent the run schedule"""
    def __init__(
        self,
        *,
        environments: collections.abc.Iterable[server_pb2.EnvironmentDefinition] | None = ...,
        machine_requirements: global___MachineRequirements | None = ...,
        function: common_pb2.SerializedObject | None = ...,
        cron: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_machine_requirements", b"_machine_requirements", "function", b"function", "machine_requirements", b"machine_requirements"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_machine_requirements", b"_machine_requirements", "cron", b"cron", "environments", b"environments", "function", b"function", "machine_requirements", b"machine_requirements"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_machine_requirements", b"_machine_requirements"]) -> typing_extensions.Literal["machine_requirements"] | None: ...

global___HostedRunCron = HostedRunCron

@typing_extensions.final
class HostedRunResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RUN_ID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    LOGS_FIELD_NUMBER: builtins.int
    RETURN_VALUE_FIELD_NUMBER: builtins.int
    run_id: builtins.str
    """Unique run id / token."""
    @property
    def status(self) -> global___HostedRunStatus:
        """Optionally the status of the current run (in terms of
        fal cloud).
        """
    @property
    def logs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.Log]:
        """The most recent logs from the run."""
    @property
    def return_value(self) -> common_pb2.SerializedObject:
        """The result of the run, if it is complete (indicated by
        status.is_complete).
        """
    def __init__(
        self,
        *,
        run_id: builtins.str = ...,
        status: global___HostedRunStatus | None = ...,
        logs: collections.abc.Iterable[common_pb2.Log] | None = ...,
        return_value: common_pb2.SerializedObject | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_return_value", b"_return_value", "_status", b"_status", "return_value", b"return_value", "status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_return_value", b"_return_value", "_status", b"_status", "logs", b"logs", "return_value", b"return_value", "run_id", b"run_id", "status", b"status"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_return_value", b"_return_value"]) -> typing_extensions.Literal["return_value"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_status", b"_status"]) -> typing_extensions.Literal["status"] | None: ...

global___HostedRunResult = HostedRunResult

@typing_extensions.final
class HostedRunStatus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _State:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[HostedRunStatus._State.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        IN_PROGRESS: HostedRunStatus._State.ValueType  # 0
        """The run is in progress."""
        SUCCESS: HostedRunStatus._State.ValueType  # 1
        """The run has completed successfully."""
        INTERNAL_FAILURE: HostedRunStatus._State.ValueType  # 2
        """The run has failed because of isolate.
        TODO: probably QUEUED, etc.
        """

    class State(_State, metaclass=_StateEnumTypeWrapper): ...
    IN_PROGRESS: HostedRunStatus.State.ValueType  # 0
    """The run is in progress."""
    SUCCESS: HostedRunStatus.State.ValueType  # 1
    """The run has completed successfully."""
    INTERNAL_FAILURE: HostedRunStatus.State.ValueType  # 2
    """The run has failed because of isolate.
    TODO: probably QUEUED, etc.
    """

    STATE_FIELD_NUMBER: builtins.int
    state: global___HostedRunStatus.State.ValueType
    """The state of the run."""
    def __init__(
        self,
        *,
        state: global___HostedRunStatus.State.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["state", b"state"]) -> None: ...

global___HostedRunStatus = HostedRunStatus

@typing_extensions.final
class MachineRequirements(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MACHINE_TYPE_FIELD_NUMBER: builtins.int
    machine_type: builtins.str
    """Machine type. It is not an enum because we want to be able
    to dynamically add new machine types without regenerating
    both the client and the server. Validation is done at the
    server side.
    """
    def __init__(
        self,
        *,
        machine_type: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["machine_type", b"machine_type"]) -> None: ...

global___MachineRequirements = MachineRequirements



from __future__ import annotations

from dataclasses import dataclass
import pkgutil
from typing import (
    Any,
    ClassVar,
    Dict,
    List,
    Optional,
)
from grpc import ChannelCredentials
import grpc

from isolate.backends import (
    BasicCallable,
)

from isolate.backends.settings import IsolateSettings
from isolate.server.definitions import (
    EnvironmentDefinition,
)
from isolate.backends.common import sha256_digest_of
from isolate.server import interface

from isolate_cloud.definitions import *
from isolate.interface import RemoteBox, BoxedEnvironment
from isolate.backends.remote import IsolateServer, IsolateServerConnection
import isolate_cloud.auth as auth

from isolate_controller import flags

@dataclass
class HostedRemoteBox(RemoteBox):
    """Run on an hosted isolate server."""
    creds: Optional[ChannelCredentials] = None 

    def wrap(
        self,
        definition: Dict[str, Any],
        settings: IsolateSettings,
    ) -> BoxedEnvironment:
        definition = definition.copy()

        # Extract the kind of environment to use.
        kind = definition.pop("kind", None)
        assert kind is not None, f"Corrupted definition: {definition}"

        target_list = [{"kind": kind, "configuration": definition}]

        # Create a remote environment.
        return BoxedEnvironment(
            FalHostedServer(host=self.host, target_environments=target_list,  creds=self.creds)
        )


@dataclass
class FalHostedServer(IsolateServer):
    BACKEND_NAME: ClassVar[str] = "hosted-isolate-server"
    creds: Optional[ChannelCredentials] = None 

    def open_connection(
        self,
        connection_key: List[EnvironmentDefinition],
    ) -> FalHostedServerConnection:
        return FalHostedServerConnection(self, self.host, connection_key, creds=self.creds)


@dataclass
class FalHostedServerConnection(IsolateServerConnection):
    creds: Optional[ChannelCredentials] = None 
    
    def _acquire_channel(self, channel_creds) -> None:
        TOKEN_KEY = "auth-token"

        root_cert = pkgutil.get_data(__name__, "ca.pem")

        class GrpcAuth(grpc.AuthMetadataPlugin):
            def __init__(self, key):
                self._key = key

            def __call__(
                self,
                context: grpc.AuthMetadataContext,
                callback: grpc.AuthMetadataPluginCallback,
            ):
                # Add token to metadata before sending
                callback(((TOKEN_KEY, self._key),), None)

        if flags.TEST_MODE:
            channel_creds = grpc.local_channel_credentials()
        else:                
            channel_creds = grpc.composite_channel_credentials(
                # Channel credentials
                grpc.ssl_channel_credentials(root_cert),
                # User credentials
                # TODO: "access_key" for now, replace with user tokens later
                grpc.metadata_call_credentials(GrpcAuth(auth.USER.access_token)),
            )

        options = (("grpc.ssl_target_name_override", "localhost"),)
        self._channel = grpc.secure_channel(self.host, channel_creds, options)


    def run(
        self,
        executable: BasicCallable,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        self._acquire_channel(self.creds)
        isolate_controller = IsolateControllerStub(self._channel)

        request = HostedRun(
            environments=self.definitions,
            function=interface.to_serialized_object(
                executable,
                method="dill",
                was_it_raised=False,
            ),
        )
        
        return_value = []
        for result in isolate_controller.Run(request):
            for raw_log in result.logs:
                log = interface.from_grpc(raw_log)
                self.log(log.message, level=log.level, source=log.source)
            if result.return_value.definition:
                return_value.append(interface.from_grpc(result.return_value))
        return return_value[0]



from typing import Optional

import os
from pathlib import Path

_DEFAULT_HOME_DIR = str(Path.home() / ".fal")
_FAL_HOME_DIR = os.getenv("FAL_HOME_DIR", _DEFAULT_HOME_DIR)
_TOKEN_FILE = "auth0_token"


def _check_dir_exist(input_location=None):
    """
    Checks if a specific directory exists, creates if not.
    In case the user didn't set a custom dir, will turn to the default home
    """
    dir = Path(_FAL_HOME_DIR).expanduser()

    if input_location:
        dir = dir / input_location

    if not dir.exists():
        dir.mkdir(parents=True)

    return dir


def _read_refresh_token(path: Path) -> Optional[str]:
    if path.exists():
        return path.read_text()


def _write_refresh_token(path: Path, access_token: str):
    path.write_text(access_token)


def load_token() -> Optional[str]:
    return _read_refresh_token(_check_dir_exist() / _TOKEN_FILE)


def save_token(token: str) -> None:
    return _write_refresh_token(_check_dir_exist() / _TOKEN_FILE, token)

def delete_token() -> None:
    path = _check_dir_exist() / _TOKEN_FILE
    path.unlink()



from google.protobuf.json_format import MessageToDict as struct_to_dict
from google.protobuf.struct_pb2 import Struct

# Imitates how includes in our protobuf files are handled.
from isolate.connections.grpc.definitions import *
from isolate.server.definitions import *

from isolate_cloud.definitions.controller_pb2 import *
from isolate_cloud.definitions.controller_pb2_grpc import (
    IsolateControllerServicer,
    IsolateControllerStub,
)
from isolate_cloud.definitions.controller_pb2_grpc import (
    add_IsolateControllerServicer_to_server as register_controller,
)



import typer

import isolate_cloud.auth as auth

cli = typer.Typer()

@cli.command()
def login(revoke: bool = False):
    if revoke:
        auth.logout()
    else:
        auth.login()

@cli.command(hidden=True)
def hello():
    """
    To test auth.
    """
    print(f"Hello, {auth.USER.info['name']}")

if __name__ == "__main__":
    cli()



from typing import Optional

from dataclasses import dataclass, field
import click

from isolate_cloud.auth import auth0
from isolate_cloud.auth import local


def login():
    token_data = auth0.login()
    local.save_token(token_data["refresh_token"])

def logout():
    refresh_token = local.load_token()
    if refresh_token is None:
        raise click.ClickException(message="No local token found")
    auth0.revoke(refresh_token)
    local.delete_token()

def _fetch_access_token() -> str:
    """
    Load the refresh token, request a new access_token (refreshing the refresh token)
    and return the access_token.

    TODO: We could do this if the access_token is expired (it lasts 1 day),
    instead of every time we invoke.
    """
    refresh_token = local.load_token()

    if refresh_token is None:
        raise click.ClickException(message="Use `isolate-cloud login` flow")

    try:
        token_data = auth0.refresh(refresh_token)
    except click.ClickException:
        local.delete_token()
        raise

    # NOTE: Auth0 Refresh Token Rotation enabled
    # So the old refresh_token is no longer valid
    local.save_token(token_data["refresh_token"])

    return f"{token_data['token_type']} {token_data['access_token']}"


@dataclass
class _User:
    _access_token: Optional[str] = field(repr=False, default=None)
    _user_info: Optional[dict] = field(repr=False, default=None)

    @property
    def info(self) -> dict:
        if self._user_info is None:
            self._user_info = auth0.get_user_info(self.access_token)

        return self._user_info

    @property
    def access_token(self) -> str:
        if self._access_token is None:
            self._access_token = _fetch_access_token()

        return self._access_token


USER = _User()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: controller.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from isolate.connections.grpc.definitions import common_pb2 as common__pb2
from isolate.server.definitions import server_pb2 as server__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x63ontroller.proto\x12\ncontroller\x1a\x0c\x63ommon.proto\x1a\x0cserver.proto\"\xbb\x01\n\tHostedRun\x12,\n\x0c\x65nvironments\x18\x01 \x03(\x0b\x32\x16.EnvironmentDefinition\x12\x42\n\x14machine_requirements\x18\x02 \x01(\x0b\x32\x1f.controller.MachineRequirementsH\x00\x88\x01\x01\x12#\n\x08\x66unction\x18\x03 \x01(\x0b\x32\x11.SerializedObjectB\x17\n\x15_machine_requirements\"\xcd\x01\n\rHostedRunCron\x12,\n\x0c\x65nvironments\x18\x01 \x03(\x0b\x32\x16.EnvironmentDefinition\x12\x42\n\x14machine_requirements\x18\x02 \x01(\x0b\x32\x1f.controller.MachineRequirementsH\x00\x88\x01\x01\x12#\n\x08\x66unction\x18\x03 \x01(\x0b\x32\x11.SerializedObject\x12\x0c\n\x04\x63ron\x18\x04 \x01(\tB\x17\n\x15_machine_requirements\"\xb1\x01\n\x0fHostedRunResult\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x30\n\x06status\x18\x02 \x01(\x0b\x32\x1b.controller.HostedRunStatusH\x00\x88\x01\x01\x12\x12\n\x04logs\x18\x03 \x03(\x0b\x32\x04.Log\x12,\n\x0creturn_value\x18\x04 \x01(\x0b\x32\x11.SerializedObjectH\x01\x88\x01\x01\x42\t\n\x07_statusB\x0f\n\r_return_value\"\x80\x01\n\x0fHostedRunStatus\x12\x30\n\x05state\x18\x01 \x01(\x0e\x32!.controller.HostedRunStatus.State\";\n\x05State\x12\x0f\n\x0bIN_PROGRESS\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\x14\n\x10INTERNAL_FAILURE\x10\x02\"+\n\x13MachineRequirements\x12\x14\n\x0cmachine_type\x18\x01 \x01(\t2R\n\x11IsolateController\x12=\n\x03Run\x12\x15.controller.HostedRun\x1a\x1b.controller.HostedRunResult\"\x00\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'controller_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HOSTEDRUN._serialized_start=61
  _HOSTEDRUN._serialized_end=248
  _HOSTEDRUNCRON._serialized_start=251
  _HOSTEDRUNCRON._serialized_end=456
  _HOSTEDRUNRESULT._serialized_start=459
  _HOSTEDRUNRESULT._serialized_end=636
  _HOSTEDRUNSTATUS._serialized_start=639
  _HOSTEDRUNSTATUS._serialized_end=767
  _HOSTEDRUNSTATUS_STATE._serialized_start=708
  _HOSTEDRUNSTATUS_STATE._serialized_end=767
  _MACHINEREQUIREMENTS._serialized_start=769
  _MACHINEREQUIREMENTS._serialized_end=812
  _ISOLATECONTROLLER._serialized_start=814
  _ISOLATECONTROLLER._serialized_end=896
# @@protoc_insertion_point(module_scope)



import click
import requests
import time

from auth0.v3.authentication.token_verifier import (
    TokenVerifier,
    AsymmetricSignatureVerifier,
)

AUTH0_CLIENT_ID = "TwXR51Vz8JbY8GUUMy6EyuVR0fTO7N4N"
AUTH0_DOMAIN = "dev-n2t1kjuo8uh0ddfg.us.auth0.com"
AUTH0_SCOPE = "openid profile email offline_access"
AUTH0_ALGORITHMS = ["RS256"]


def login() -> dict:
    """
    Runs the device authorization flow and stores the user object in memory
    """
    device_code_payload = {"client_id": AUTH0_CLIENT_ID, "scope": AUTH0_SCOPE}
    device_code_response = requests.post(
        "https://{}/oauth/device/code".format(AUTH0_DOMAIN), data=device_code_payload
    )

    if device_code_response.status_code != 200:
        raise click.ClickException("Error generating the device code")

    print("Device code successful")
    device_code_data = device_code_response.json()
    print(
        "1. On your computer or mobile device navigate to: ",
        device_code_data["verification_uri_complete"],
    )
    print("2. Enter the following code: ", device_code_data["user_code"])

    token_payload = {
        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
        "device_code": device_code_data["device_code"],
        "client_id": AUTH0_CLIENT_ID,
    }

    while True:
        print("Checking if the user completed the flow...")
        token_response = requests.post(
            "https://{}/oauth/token".format(AUTH0_DOMAIN), data=token_payload
        )

        token_data = token_response.json()
        if token_response.status_code == 200:
            print("Authenticated!")

            validate_token(token_data["id_token"])

            return token_data

        elif token_data["error"] not in ("authorization_pending", "slow_down"):
            raise click.ClickException(token_data["error_description"])

        else:
            time.sleep(device_code_data["interval"])


def refresh(token: str) -> dict:
    token_payload = {
        "grant_type": "refresh_token",
        "client_id": AUTH0_CLIENT_ID,
        "refresh_token": token,
    }

    token_response = requests.post(
        "https://{}/oauth/token".format(AUTH0_DOMAIN), data=token_payload
    )

    token_data = token_response.json()
    if token_response.status_code == 200:
        # DEBUG: print("Authenticated!")

        validate_token(token_data["id_token"])

        return token_data
    else:
        raise click.ClickException(token_data["error_description"])

def revoke(token: str):
    token_payload = {
        "client_id": AUTH0_CLIENT_ID,
        "token": token,
    }

    token_response = requests.post(
        "https://{}/oauth/revoke".format(AUTH0_DOMAIN), data=token_payload
    )

    if token_response.status_code != 200:
        token_data = token_response.json()
        raise click.ClickException(token_data["error_description"])


def get_user_info(access_token: str) -> dict:
    userinfo_response = requests.post(
        "https://{}/userinfo".format(AUTH0_DOMAIN),
        headers={"Authorization": access_token},
    )

    return userinfo_response.json()


def validate_token(id_token):
    """
    Verify the token and its precedence

    :param id_token:
    """
    jwks_url = "https://{}/.well-known/jwks.json".format(AUTH0_DOMAIN)
    issuer = "https://{}/".format(AUTH0_DOMAIN)

    sv = AsymmetricSignatureVerifier(jwks_url)
    tv = TokenVerifier(signature_verifier=sv, issuer=issuer, audience=AUTH0_CLIENT_ID)
    tv.verify(id_token)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['isolate_cloud', 'isolate_cloud.auth', 'isolate_cloud.definitions']

package_data = \
{'': ['*']}

install_requires = \
['auth0-python>=3.24.0,<4.0.0',
 'grpcio>=1.50.0,<2.0.0',
 'isolate>=0.5.0,<0.6.0',
 'requests>=2.28.1,<3.0.0',
 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['isolate-cloud = isolate_cloud.cli:cli']}

setup_kwargs = {
    'name': 'isolate-cloud',
    'version': '0.5.0',
    'description': 'SDK and cli for the fal isolate Cloud service',
    'long_description': '',
    'author': 'Features & Labels',
    'author_email': 'hello@fal.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
